(*
** 8/1/96 - Created by calix@together.net
** Inspired by Brendan V. Delumpa
**
** TRollup is a control that will Roll up its owner form.
**
** I obtained all the information required to create this
** component from my TSnapToGrid component.
**
*)

unit cRollup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TRollup = class(TComponent)
  private
    FEnabled: Boolean;

  protected
    FOldWndProc:Pointer;
    FExtWndProc:TFarProc;
    FMinHeight: Integer;

    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    Procedure ExtWndProc(Var Msg:TMessage);
    procedure SetEnabled(Value: Boolean);
    function GetRolledUp: Boolean;
    procedure SetRolledUp(Value: Boolean);
    procedure SetMinHeight(Value: Integer);

  published
    property isEnabled: Boolean read FEnabled write SetEnabled;
    property isRolledUp: Boolean read GetRolledUp write SetRolledUp;
    property MinHeight: Integer read FMinHeight write SetMinHeight;

  end;

procedure Register;

implementation

{--------------------------------------------------------}
constructor TRollup.Create;
begin
  inherited;
  { get old OnMessage and process after were done with it }
  FExtWndProc := MakeObjectInstance(ExtWndProc);
  FEnabled := FALSE;
  FMinHeight := 0;
end;

{-------------------------------------------------------}
destructor TRollup.Destroy;
begin
  { restore old handler }
  SetEnabled(FALSE);
  inherited;
end;

{-------------------------------------------------------}
Procedure TRollup.SetEnabled(Value: Boolean);
begin
  if (FEnabled = Value) then exit;
  if (Owner is TForm) then begin
    FEnabled := Value;
    if Value then
      FOldWndProc:= Pointer(
        SetWindowLong(TForm(Owner).Handle, GWL_WNDPROC, LongInt(FExtWndProc)))
    else
      SetWindowLong(TForm(Owner).Handle, GWL_WNDPROC, LongInt(FOldWndProc));
  end;
end;

{-------------------------------------------------------}
function TRollup.GetRolledUp: Boolean;
begin
  Result := (Owner is TForm) AND (FMinHeight >= (Owner as TForm).ClientHeight);
end;

{-------------------------------------------------------}
procedure TRollup.SetRolledUp(Value: Boolean);
const
  FOldHeight : Integer = 0;
begin
  if (owner is TForm) then begin
    with (owner as TForm) do begin
      if (Value) then begin
        FOldHeight := ClientHeight;
        ClientHeight := FMinHeight;
        FMinHeight := ClientHeight;
      end else begin
        if (FOldHeight < FMinHeight) then FOldHeight := FMinHeight;
        ClientHeight := FOldHeight;
      end;
    end;
  end;
end;

{-------------------------------------------------------}
procedure TRollup.SetMinHeight(Value: Integer);
begin
  if (Value > 0) then FMinHeight := Value;
end;

{-------------------------------------------------------}
Procedure TRollup.ExtWndProc(Var Msg:TMessage);
begin
  with Msg do begin
    // this particular combination will rollup the form when the
    // user right clicks the title bar.
    if (msg = WM_NCRBUTTONDOWN) AND (wParam = HTCAPTION) then begin
      SetRolledUp(NOT GetRolledUp);
      exit;
    end;
    Result := CallWindowProc(FOldWndProc, TForm(Owner).Handle, Msg, wParam, lParam);
  end;
end;

{--------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('Coretech', [TRollup]);
end;

end.
